@@ -8,26 +8,85 @@ import subprocess
 import sys
 import threading
 
+from typing import Optional, List
+
 from homeassistant.const import (
     __version__,
     EVENT_HOMEASSISTANT_START,
     REQUIRED_PYTHON_VER,
+    REQUIRED_PYTHON_VER_WIN,
     RESTART_EXIT_CODE,
 )
+from homeassistant.util.async import run_callback_threadsafe
 
 
-def validate_python():
+def attempt_use_uvloop():
+    """Attempt to use uvloop."""
+    import asyncio
+
+    try:
+        import uvloop
+        asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
+    except ImportError:
+        pass
+
+
+def monkey_patch_asyncio():
+    """Replace weakref.WeakSet to address Python 3 bug.
+
+    Under heavy threading operations that schedule calls into
+    the asyncio event loop, Task objects are created. Due to
+    a bug in Python, GC may have an issue when switching between
+    the threads and objects with __del__ (which various components
+    in HASS have).
+
+    This monkey-patch removes the weakref.Weakset, and replaces it
+    with an object that ignores the only call utilizing it (the
+    Task.__init__ which calls _all_tasks.add(self)). It also removes
+    the __del__ which could trigger the future objects __del__ at
+    unpredictable times.
+
+    The side-effect of this manipulation of the Task is that
+    Task.all_tasks() is no longer accurate, and there will be no
+    warning emitted if a Task is GC'd while in use.
+
+    On Python 3.6, after the bug is fixed, this monkey-patch can be
+    disabled.
+
+    See https://bugs.python.org/issue26617 for details of the Python
+    bug.
+    """
+    # pylint: disable=no-self-use, protected-access, bare-except
+    import asyncio.tasks
+
+    class IgnoreCalls:
+        """Ignore add calls."""
+
+        def add(self, other):
+            """No-op add."""
+            return
+
+    asyncio.tasks.Task._all_tasks = IgnoreCalls()
+    try:
+        del asyncio.tasks.Task.__del__
+    except:
+        pass
+
+
+def validate_python() -> None:
     """Validate we're running the right Python version."""
-    major, minor = sys.version_info[:2]
-    req_major, req_minor = REQUIRED_PYTHON_VER
-
-    if major < req_major or (major == req_major and minor < req_minor):
-        print("Home Assistant requires at least Python {}.{}".format(
-            req_major, req_minor))
+    if sys.platform == "win32" and \
+       sys.version_info[:3] < REQUIRED_PYTHON_VER_WIN:
+        print("Home Assistant requires at least Python {}.{}.{}".format(
+            *REQUIRED_PYTHON_VER_WIN))
+        sys.exit(1)
+    elif sys.version_info[:3] < REQUIRED_PYTHON_VER:
+        print("Home Assistant requires at least Python {}.{}.{}".format(
+            *REQUIRED_PYTHON_VER))
         sys.exit(1)
 
 
-def ensure_config_path(config_dir):
+def ensure_config_path(config_dir: str) -> None:
     """Validate the configuration directory."""
     import homeassistant.config as config_util
     lib_dir = os.path.join(config_dir, 'deps')
@@ -56,7 +115,7 @@ def ensure_config_path(config_dir):
             sys.exit(1)
 
 
-def ensure_config_file(config_dir):
+def ensure_config_file(config_dir: str) -> str:
     """Ensure configuration file exists."""
     import homeassistant.config as config_util
     config_path = config_util.ensure_config_exists(config_dir)
@@ -68,7 +127,7 @@ def ensure_config_file(config_dir):
     return config_path
 
 
-def get_arguments():
+def get_arguments() -> argparse.Namespace:
     """Get parsed passed in arguments."""
     import homeassistant.config as config_util
     parser = argparse.ArgumentParser(
@@ -125,12 +184,12 @@ def get_arguments():
 
     arguments = parser.parse_args()
     if os.name != "posix" or arguments.debug or arguments.runner:
-        arguments.daemon = False
+        setattr(arguments, 'daemon', False)
 
     return arguments
 
 
-def daemonize():
+def daemonize() -> None:
     """Move current process to daemon process."""
     # Create first fork
     pid = os.fork()
@@ -155,7 +214,7 @@ def daemonize():
     os.dup2(outfd.fileno(), sys.stderr.fileno())
 
 
-def check_pid(pid_file):
+def check_pid(pid_file: str) -> None:
     """Check that HA is not already running."""
     # Check pid file
     try:
@@ -177,7 +236,7 @@ def check_pid(pid_file):
     sys.exit(1)
 
 
-def write_pid(pid_file):
+def write_pid(pid_file: str) -> None:
     """Create a PID File."""
     pid = os.getpid()
     try:
@@ -187,7 +246,7 @@ def write_pid(pid_file):
         sys.exit(1)
 
 
-def closefds_osx(min_fd, max_fd):
+def closefds_osx(min_fd: int, max_fd: int) -> None:
     """Make sure file descriptors get closed when we restart.
 
     We cannot call close on guarded fds, and we cannot easily test which fds
@@ -205,24 +264,28 @@ def closefds_osx(min_fd, max_fd):
             pass
 
 
-def cmdline():
+def cmdline() -> List[str]:
     """Collect path and arguments to re-execute the current hass instance."""
-    if sys.argv[0].endswith('/__main__.py'):
+    if sys.argv[0].endswith(os.path.sep + '__main__.py'):
         modulepath = os.path.dirname(sys.argv[0])
         os.environ['PYTHONPATH'] = os.path.dirname(modulepath)
-    return [sys.executable] + [arg for arg in sys.argv if arg != '--daemon']
+        return [sys.executable] + [arg for arg in sys.argv if
+                                   arg != '--daemon']
+    else:
+        return [arg for arg in sys.argv if arg != '--daemon']
 
 
-def setup_and_run_hass(config_dir, args):
+def setup_and_run_hass(config_dir: str,
+                       args: argparse.Namespace) -> Optional[int]:
     """Setup HASS and run."""
     from homeassistant import bootstrap
 
     # Run a simple daemon runner process on Windows to handle restarts
     if os.name == 'nt' and '--runner' not in sys.argv:
-        args = cmdline() + ['--runner']
+        nt_args = cmdline() + ['--runner']
         while True:
             try:
-                subprocess.check_call(args)
+                subprocess.check_call(nt_args)
                 sys.exit(0)
             except subprocess.CalledProcessError as exc:
                 if exc.returncode != RESTART_EXIT_CODE:
@@ -244,7 +307,7 @@ def setup_and_run_hass(config_dir, args):
             log_rotate_days=args.log_rotate_days)
 
     if hass is None:
-        return
+        return None
 
     if args.open_ui:
         def open_browser(event):
@@ -253,15 +316,16 @@ def setup_and_run_hass(config_dir, args):
                 import webbrowser
                 webbrowser.open(hass.config.api.base_url)
 
-        hass.bus.listen_once(EVENT_HOMEASSISTANT_START, open_browser)
+        run_callback_threadsafe(
+            hass.loop,
+            hass.bus.async_listen_once,
+            EVENT_HOMEASSISTANT_START, open_browser
+        )
 
-    hass.start()
-    exit_code = int(hass.block_till_stopped())
-
-    return exit_code
+    return hass.start()
 
 
-def try_to_restart():
+def try_to_restart() -> None:
     """Attempt to clean up state and start a new homeassistant instance."""
     # Things should be mostly shut down already at this point, now just try
     # to clean up things that may have been left behind.
@@ -271,7 +335,7 @@ def try_to_restart():
     # thread left (which is us). Nothing we really do with it, but it might be
     # useful when debugging shutdown/restart issues.
     try:
-        nthreads = sum(thread.isAlive() and not thread.isDaemon()
+        nthreads = sum(thread.is_alive() and not thread.daemon
                        for thread in threading.enumerate())
         if nthreads > 1:
             sys.stderr.write(
@@ -303,10 +367,15 @@ def try_to_restart():
     os.execv(args[0], args)
 
 
-def main():
+def main() -> int:
     """Start Home Assistant."""
     validate_python()
 
+    attempt_use_uvloop()
+
+    if sys.version_info[:3] < (3, 5, 3):
+        monkey_patch_asyncio()
+
     args = get_arguments()
 
     if args.script is not None:
