@@ -2,16 +2,18 @@
 Offer sun based automation rules.
 
 For more details about this automation rule, please refer to the documentation
-at https://home-assistant.io/components/automation/#sun-trigger
+at https://home-assistant.io/docs/automation/trigger/#sun-trigger
 """
+import asyncio
 from datetime import timedelta
 import logging
 
 import voluptuous as vol
 
+from homeassistant.core import callback
 from homeassistant.const import (
     CONF_EVENT, CONF_OFFSET, CONF_PLATFORM, SUN_EVENT_SUNRISE)
-from homeassistant.helpers.event import track_sunrise, track_sunset
+from homeassistant.helpers.event import async_track_sunrise, async_track_sunset
 import homeassistant.helpers.config_validation as cv
 
 DEPENDENCIES = ['sun']
@@ -25,14 +27,16 @@ TRIGGER_SCHEMA = vol.Schema({
 })
 
 
-def trigger(hass, config, action):
+@asyncio.coroutine
+def async_trigger(hass, config, action):
     """Listen for events based on configuration."""
     event = config.get(CONF_EVENT)
     offset = config.get(CONF_OFFSET)
 
+    @callback
     def call_action():
         """Call action with right context."""
-        action({
+        hass.async_run_job(action, {
             'trigger': {
                 'platform': 'sun',
                 'event': event,
@@ -42,8 +46,6 @@ def trigger(hass, config, action):
 
     # Do something to call action
     if event == SUN_EVENT_SUNRISE:
-        track_sunrise(hass, call_action, offset)
+        return async_track_sunrise(hass, call_action, offset)
     else:
-        track_sunset(hass, call_action, offset)
-
-    return True
+        return async_track_sunset(hass, call_action, offset)
