@@ -6,37 +6,39 @@ This will return a request id that has to be used for future calls.
 A callback has to be provided to `request_config` which will be called when
 the user has submitted configuration information.
 """
+import asyncio
 import logging
 
-from homeassistant.const import EVENT_TIME_CHANGED, ATTR_FRIENDLY_NAME
+from homeassistant.const import EVENT_TIME_CHANGED, ATTR_FRIENDLY_NAME, \
+    ATTR_ENTITY_PICTURE
 from homeassistant.helpers.entity import generate_entity_id
 
-DOMAIN = "configurator"
-ENTITY_ID_FORMAT = DOMAIN + ".{}"
-
-SERVICE_CONFIGURE = "configure"
-
-STATE_CONFIGURE = "configure"
-STATE_CONFIGURED = "configured"
-
-ATTR_LINK_NAME = "link_name"
-ATTR_LINK_URL = "link_url"
-ATTR_CONFIGURE_ID = "configure_id"
-ATTR_DESCRIPTION = "description"
-ATTR_DESCRIPTION_IMAGE = "description_image"
-ATTR_SUBMIT_CAPTION = "submit_caption"
-ATTR_FIELDS = "fields"
-ATTR_ERRORS = "errors"
-
-_REQUESTS = {}
-_INSTANCES = {}
 _LOGGER = logging.getLogger(__name__)
+_REQUESTS = {}
+_KEY_INSTANCE = 'configurator'
+
+ATTR_CONFIGURE_ID = 'configure_id'
+ATTR_DESCRIPTION = 'description'
+ATTR_DESCRIPTION_IMAGE = 'description_image'
+ATTR_ERRORS = 'errors'
+ATTR_FIELDS = 'fields'
+ATTR_LINK_NAME = 'link_name'
+ATTR_LINK_URL = 'link_url'
+ATTR_SUBMIT_CAPTION = 'submit_caption'
+
+DOMAIN = 'configurator'
+
+ENTITY_ID_FORMAT = DOMAIN + '.{}'
+
+SERVICE_CONFIGURE = 'configure'
+STATE_CONFIGURE = 'configure'
+STATE_CONFIGURED = 'configured'
 
 
-# pylint: disable=too-many-arguments
 def request_config(
         hass, name, callback, description=None, description_image=None,
-        submit_caption=None, fields=None, link_name=None, link_url=None):
+        submit_caption=None, fields=None, link_name=None, link_url=None,
+        entity_picture=None):
     """Create a new request for configuration.
 
     Will return an ID to be used for sequent calls.
@@ -46,7 +48,7 @@ def request_config(
     request_id = instance.request_config(
         name, callback,
         description, description_image, submit_caption,
-        fields, link_name, link_url)
+        fields, link_name, link_url, entity_picture)
 
     _REQUESTS[request_id] = instance
 
@@ -71,22 +73,20 @@ def request_done(request_id):
         pass
 
 
-def setup(hass, config):
+@asyncio.coroutine
+def async_setup(hass, config):
     """Setup the configurator component."""
     return True
 
 
 def _get_instance(hass):
     """Get an instance per hass object."""
-    try:
-        return _INSTANCES[hass]
-    except KeyError:
-        _INSTANCES[hass] = Configurator(hass)
+    instance = hass.data.get(_KEY_INSTANCE)
 
-        if DOMAIN not in hass.config.components:
-            hass.config.components.append(DOMAIN)
+    if instance is None:
+        instance = hass.data[_KEY_INSTANCE] = Configurator(hass)
 
-        return _INSTANCES[hass]
+    return instance
 
 
 class Configurator(object):
@@ -100,11 +100,10 @@ class Configurator(object):
         hass.services.register(
             DOMAIN, SERVICE_CONFIGURE, self.handle_service_call)
 
-    # pylint: disable=too-many-arguments
     def request_config(
             self, name, callback,
             description, description_image, submit_caption,
-            fields, link_name, link_url):
+            fields, link_name, link_url, entity_picture):
         """Setup a request for configuration."""
         entity_id = generate_entity_id(ENTITY_ID_FORMAT, name, hass=self.hass)
 
@@ -119,6 +118,7 @@ class Configurator(object):
             ATTR_CONFIGURE_ID: request_id,
             ATTR_FIELDS: fields,
             ATTR_FRIENDLY_NAME: name,
+            ATTR_ENTITY_PICTURE: entity_picture,
         }
 
         data.update({
