@@ -8,27 +8,38 @@ import logging
 import os
 from datetime import timedelta
 
+import voluptuous as vol
+
+import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers import discovery
 from homeassistant.const import CONF_API_KEY
 from homeassistant.loader import get_component
 from homeassistant.util import Throttle
 
-DOMAIN = "ecobee"
-NETWORK = None
-HOLD_TEMP = 'hold_temp'
-
 REQUIREMENTS = [
     'https://github.com/nkgilley/python-ecobee-api/archive/'
-    '4856a704670c53afe1882178a89c209b5f98533d.zip#python-ecobee==0.0.6']
+    'a4496b293956b2eac285305136a62ac78bef510d.zip#python-ecobee==0.0.7']
 
+_CONFIGURING = {}
 _LOGGER = logging.getLogger(__name__)
 
+CONF_HOLD_TEMP = 'hold_temp'
+
+DOMAIN = 'ecobee'
+
 ECOBEE_CONFIG_FILE = 'ecobee.conf'
-_CONFIGURING = {}
 
-# Return cached results if last scan was less then this time ago.
 MIN_TIME_BETWEEN_UPDATES = timedelta(seconds=180)
 
+NETWORK = None
+
+CONFIG_SCHEMA = vol.Schema({
+    DOMAIN: vol.Schema({
+        vol.Optional(CONF_API_KEY): cv.string,
+        vol.Optional(CONF_HOLD_TEMP, default=False): cv.boolean
+    })
+}, extra=vol.ALLOW_EXTRA)
+
 
 def request_configuration(network, hass, config):
     """Request configuration steps from the user."""
@@ -67,14 +78,14 @@ def setup_ecobee(hass, network, config):
         configurator = get_component('configurator')
         configurator.request_done(_CONFIGURING.pop('ecobee'))
 
-    hold_temp = config[DOMAIN].get(HOLD_TEMP, False)
+    hold_temp = config[DOMAIN].get(CONF_HOLD_TEMP)
 
-    discovery.load_platform(hass, 'thermostat', DOMAIN,
+    discovery.load_platform(hass, 'climate', DOMAIN,
                             {'hold_temp': hold_temp}, config)
     discovery.load_platform(hass, 'sensor', DOMAIN, {}, config)
+    discovery.load_platform(hass, 'binary_sensor', DOMAIN, {}, config)
 
 
-# pylint: disable=too-few-public-methods
 class EcobeeData(object):
     """Get the latest data and update the states."""
 
@@ -87,7 +98,7 @@ class EcobeeData(object):
     def update(self):
         """Get the latest data from pyecobee."""
         self.ecobee.update()
-        _LOGGER.info("ecobee data updated successfully.")
+        _LOGGER.info("Ecobee data updated successfully")
 
 
 def setup(hass, config):
@@ -106,9 +117,6 @@ def setup(hass, config):
 
     # Create ecobee.conf if it doesn't exist
     if not os.path.isfile(hass.config.path(ECOBEE_CONFIG_FILE)):
-        if config[DOMAIN].get(CONF_API_KEY) is None:
-            _LOGGER.error("No ecobee api_key found in config.")
-            return
         jsonconfig = {"API_KEY": config[DOMAIN].get(CONF_API_KEY)}
         config_from_file(hass.config.path(ECOBEE_CONFIG_FILE), jsonconfig)
 
