@@ -10,8 +10,10 @@ import logging
 # pylint: disable=import-error
 from threading import Timer
 from homeassistant.components.light import ATTR_BRIGHTNESS, ATTR_COLOR_TEMP, \
-    ATTR_RGB_COLOR, DOMAIN, Light
+    ATTR_RGB_COLOR, ATTR_TRANSITION, SUPPORT_BRIGHTNESS, SUPPORT_COLOR_TEMP, \
+    SUPPORT_RGB_COLOR, SUPPORT_TRANSITION, DOMAIN, Light
 from homeassistant.components import zwave
+from homeassistant.components.zwave import async_setup_platform  # noqa # pylint: disable=unused-import
 from homeassistant.const import STATE_OFF, STATE_ON
 from homeassistant.util.color import HASS_COLOR_MAX, HASS_COLOR_MIN, \
     color_temperature_mired_to_kelvin, color_temperature_to_rgb, \
@@ -42,90 +44,97 @@ TEMP_WARM_HASS = (HASS_COLOR_MAX - HASS_COLOR_MIN) / 3 * 2 + HASS_COLOR_MIN
 TEMP_COLD_HASS = (HASS_COLOR_MAX - HASS_COLOR_MIN) / 3 + HASS_COLOR_MIN
 
 
-def setup_platform(hass, config, add_devices, discovery_info=None):
-    """Find and add Z-Wave lights."""
-    if discovery_info is None or zwave.NETWORK is None:
-        return
+def get_device(node, values, node_config, **kwargs):
+    """Create zwave entity device."""
+    name = '{}.{}'.format(DOMAIN, zwave.object_id(values.primary))
+    refresh = node_config.get(zwave.CONF_REFRESH_VALUE)
+    delay = node_config.get(zwave.CONF_REFRESH_DELAY)
+    _LOGGER.debug('name=%s node_config=%s CONF_REFRESH_VALUE=%s'
+                  ' CONF_REFRESH_DELAY=%s', name, node_config,
+                  refresh, delay)
 
-    node = zwave.NETWORK.nodes[discovery_info[zwave.ATTR_NODE_ID]]
-    value = node.values[discovery_info[zwave.ATTR_VALUE_ID]]
-
-    if value.command_class != zwave.COMMAND_CLASS_SWITCH_MULTILEVEL:
-        return
-    if value.type != zwave.TYPE_BYTE:
-        return
-    if value.genre != zwave.GENRE_USER:
-        return
-
-    value.set_change_verified(False)
-
-    if node.has_command_class(zwave.COMMAND_CLASS_COLOR):
-        try:
-            add_devices([ZwaveColorLight(value)])
-        except ValueError as exception:
-            _LOGGER.warning(
-                "Error initializing as color bulb: %s "
-                "Initializing as standard dimmer.", exception)
-            add_devices([ZwaveDimmer(value)])
+    if node.has_command_class(zwave.const.COMMAND_CLASS_SWITCH_COLOR):
+        return ZwaveColorLight(values, refresh, delay)
     else:
-        add_devices([ZwaveDimmer(value)])
+        return ZwaveDimmer(values, refresh, delay)
 
 
 def brightness_state(value):
     """Return the brightness and state."""
     if value.data > 0:
-        return (value.data / 99) * 255, STATE_ON
+        return round((value.data / 99) * 255, 0), STATE_ON
     else:
-        return 255, STATE_OFF
+        return 0, STATE_OFF
+
+
+def ct_to_rgb(temp):
+    """Convert color temperature (mireds) to RGB."""
+    colorlist = list(
+        color_temperature_to_rgb(color_temperature_mired_to_kelvin(temp)))
+    return [int(val) for val in colorlist]
 
 
 class ZwaveDimmer(zwave.ZWaveDeviceEntity, Light):
     """Representation of a Z-Wave dimmer."""
 
-    # pylint: disable=too-many-arguments
-    def __init__(self, value):
+    def __init__(self, values, refresh, delay):
         """Initialize the light."""
-        from openzwave.network import ZWaveNetwork
-        from pydispatch import dispatcher
-
-        zwave.ZWaveDeviceEntity.__init__(self, value, DOMAIN)
+        zwave.ZWaveDeviceEntity.__init__(self, values, DOMAIN)
         self._brightness = None
         self._state = None
-        self.update_properties()
+        self._supported_features = None
+        self._delay = delay
+        self._refresh_value = refresh
+        self._zw098 = None
+
+        # Enable appropriate workaround flags for our device
+        # Make sure that we have values for the key before converting to int
+        if (self.node.manufacturer_id.strip() and
+                self.node.product_id.strip()):
+            specific_sensor_key = (int(self.node.manufacturer_id, 16),
+                                   int(self.node.product_id, 16))
+            if specific_sensor_key in DEVICE_MAPPINGS:
+                if DEVICE_MAPPINGS[specific_sensor_key] == WORKAROUND_ZW098:
+                    _LOGGER.debug("AEOTEC ZW098 workaround enabled")
+                    self._zw098 = 1
 
         # Used for value change event handling
         self._refreshing = False
         self._timer = None
-
-        dispatcher.connect(
-            self._value_changed, ZWaveNetwork.SIGNAL_VALUE_CHANGED)
+        _LOGGER.debug('self._refreshing=%s self.delay=%s',
+                      self._refresh_value, self._delay)
+        self.value_added()
+        self.update_properties()
 
     def update_properties(self):
         """Update internal properties based on zwave values."""
         # Brightness
-        self._brightness, self._state = brightness_state(self._value)
+        self._brightness, self._state = brightness_state(self.values.primary)
 
-    def _value_changed(self, value):
-        """Called when a value has changed on the network."""
-        if self._value.value_id != value.value_id:
-            return
+    def value_added(self):
+        """Called when a new value is added to this entity."""
+        self._supported_features = SUPPORT_BRIGHTNESS
+        if self.values.dimming_duration is not None:
+            self._supported_features |= SUPPORT_TRANSITION
 
-        if self._refreshing:
-            self._refreshing = False
-            self.update_properties()
-        else:
-            def _refresh_value():
-                """Used timer callback for delayed value refresh."""
-                self._refreshing = True
-                self._value.refresh()
+    def value_changed(self):
+        """Called when a value for this entity's node has changed."""
+        if self._refresh_value:
+            if self._refreshing:
+                self._refreshing = False
+            else:
+                def _refresh_value():
+                    """Used timer callback for delayed value refresh."""
+                    self._refreshing = True
+                    self.values.primary.refresh()
 
-            if self._timer is not None and self._timer.isAlive():
-                self._timer.cancel()
+                if self._timer is not None and self._timer.isAlive():
+                    self._timer.cancel()
 
-            self._timer = Timer(2, _refresh_value)
-            self._timer.start()
-
-        self.update_ha_state()
+                self._timer = Timer(self._delay, _refresh_value)
+                self._timer.start()
+                return
+        super().value_changed()
 
     @property
     def brightness(self):
@@ -137,85 +146,98 @@ class ZwaveDimmer(zwave.ZWaveDeviceEntity, Light):
         """Return true if device is on."""
         return self._state == STATE_ON
 
+    @property
+    def supported_features(self):
+        """Flag supported features."""
+        return self._supported_features
+
+    def _set_duration(self, **kwargs):
+        """Set the transition time for the brightness value.
+
+        Zwave Dimming Duration values:
+        0x00      = instant
+        0x01-0x7F = 1 second to 127 seconds
+        0x80-0xFE = 1 minute to 127 minutes
+        0xFF      = factory default
+        """
+        if self.values.dimming_duration is None:
+            if ATTR_TRANSITION in kwargs:
+                _LOGGER.debug("Dimming not supported by %s.", self.entity_id)
+            return
+
+        if ATTR_TRANSITION not in kwargs:
+            self.values.dimming_duration.data = 0xFF
+            return
+
+        transition = kwargs[ATTR_TRANSITION]
+        if transition <= 127:
+            self.values.dimming_duration.data = int(transition)
+        elif transition > 7620:
+            self.values.dimming_duration.data = 0xFE
+            _LOGGER.warning("Transition clipped to 127 minutes for %s.",
+                            self.entity_id)
+        else:
+            minutes = int(transition / 60)
+            _LOGGER.debug("Transition rounded to %d minutes for %s.",
+                          minutes, self.entity_id)
+            self.values.dimming_duration.data = minutes + 0x7F
+
     def turn_on(self, **kwargs):
         """Turn the device on."""
-        if ATTR_BRIGHTNESS in kwargs:
-            self._brightness = kwargs[ATTR_BRIGHTNESS]
+        self._set_duration(**kwargs)
 
         # Zwave multilevel switches use a range of [0, 99] to control
-        # brightness.
-        brightness = int((self._brightness / 255) * 99)
+        # brightness. Level 255 means to set it to previous value.
+        if ATTR_BRIGHTNESS in kwargs:
+            self._brightness = kwargs[ATTR_BRIGHTNESS]
+            brightness = int((self._brightness / 255) * 99)
+        else:
+            brightness = 255
 
-        if self._value.node.set_dimmer(self._value.value_id, brightness):
+        if self.node.set_dimmer(self.values.primary.value_id, brightness):
             self._state = STATE_ON
 
     def turn_off(self, **kwargs):
         """Turn the device off."""
-        if self._value.node.set_dimmer(self._value.value_id, 0):
+        self._set_duration(**kwargs)
+
+        if self.node.set_dimmer(self.values.primary.value_id, 0):
             self._state = STATE_OFF
 
 
-def ct_to_rgb(temp):
-    """Convert color temperature (mireds) to RGB."""
-    colorlist = list(
-        color_temperature_to_rgb(color_temperature_mired_to_kelvin(temp)))
-    return [int(val) for val in colorlist]
-
-
 class ZwaveColorLight(ZwaveDimmer):
     """Representation of a Z-Wave color changing light."""
 
-    def __init__(self, value):
+    def __init__(self, values, refresh, delay):
         """Initialize the light."""
-        self._value_color = None
-        self._value_color_channels = None
         self._color_channels = None
         self._rgb = None
         self._ct = None
-        self._zw098 = None
 
-        # Here we attempt to find a zwave color value with the same instance
-        # id as the dimmer value. Currently zwave nodes that change colors
-        # only include one dimmer and one color command, but this will
-        # hopefully provide some forward compatibility for new devices that
-        # have multiple color changing elements.
-        for value_color in value.node.get_rgbbulbs().values():
-            if value.instance == value_color.instance:
-                self._value_color = value_color
+        super().__init__(values, refresh, delay)
 
-        if self._value_color is None:
-            raise ValueError("No matching color command found.")
+    def value_added(self):
+        """Called when a new value is added to this entity."""
+        super().value_added()
 
-        for value_color_channels in value.node.get_values(
-                class_id=zwave.COMMAND_CLASS_COLOR, genre='System',
-                type="Int").values():
-            self._value_color_channels = value_color_channels
-
-        if self._value_color_channels is None:
-            raise ValueError("Color Channels not found.")
-
-        # Make sure that we have values for the key before converting to int
-        if (value.node.manufacturer_id.strip() and
-                value.node.product_id.strip()):
-            specific_sensor_key = (int(value.node.manufacturer_id, 16),
-                                   int(value.node.product_id, 16))
-
-            if specific_sensor_key in DEVICE_MAPPINGS:
-                if DEVICE_MAPPINGS[specific_sensor_key] == WORKAROUND_ZW098:
-                    _LOGGER.debug("AEOTEC ZW098 workaround enabled")
-                    self._zw098 = 1
-
-        super().__init__(value)
+        self._supported_features |= SUPPORT_RGB_COLOR
+        if self._zw098:
+            self._supported_features |= SUPPORT_COLOR_TEMP
 
     def update_properties(self):
         """Update internal properties based on zwave values."""
         super().update_properties()
 
+        if self.values.color is None:
+            return
+        if self.values.color_channels is None:
+            return
+
         # Color Channels
-        self._color_channels = self._value_color_channels.data
+        self._color_channels = self.values.color_channels.data
 
         # Color Data String
-        data = self._value_color.data
+        data = self.values.color.data
 
         # RGB is always present in the openzwave color data string.
         self._rgb = [
@@ -288,10 +310,10 @@ class ZwaveColorLight(ZwaveDimmer):
             if self._zw098:
                 if kwargs[ATTR_COLOR_TEMP] > TEMP_MID_HASS:
                     self._ct = TEMP_WARM_HASS
-                    rgbw = b'#000000FF00'
+                    rgbw = b'#000000ff00'
                 else:
                     self._ct = TEMP_COLD_HASS
-                    rgbw = b'#00000000FF'
+                    rgbw = b'#00000000ff'
 
         elif ATTR_RGB_COLOR in kwargs:
             self._rgb = kwargs[ATTR_RGB_COLOR]
@@ -308,9 +330,7 @@ class ZwaveColorLight(ZwaveDimmer):
                     rgbw += format(colorval, '02x').encode('utf-8')
                 rgbw += b'0000'
 
-        if rgbw is None:
-            _LOGGER.warning("rgbw string was not generated for turn_on")
-        else:
-            self._value_color.node.set_rgbw(self._value_color.value_id, rgbw)
+        if rgbw and self.values.color:
+            self.values.color.data = rgbw
 
         super().turn_on(**kwargs)
