@@ -4,7 +4,9 @@ Component to interface with various locks that can be controlled remotely.
 For more details about this component, please refer to the documentation
 at https://home-assistant.io/components/lock/
 """
+import asyncio
 from datetime import timedelta
+import functools as ft
 import logging
 import os
 
@@ -21,7 +23,8 @@ from homeassistant.const import (
 from homeassistant.components import group
 
 DOMAIN = 'lock'
-SCAN_INTERVAL = 30
+SCAN_INTERVAL = timedelta(seconds=30)
+ATTR_CHANGED_BY = 'changed_by'
 
 GROUP_NAME_ALL_LOCKS = 'all locks'
 ENTITY_ID_ALL_LOCKS = group.ENTITY_ID_FORMAT.format('all_locks')
@@ -66,41 +69,65 @@ def unlock(hass, entity_id=None, code=None):
     hass.services.call(DOMAIN, SERVICE_UNLOCK, data)
 
 
-def setup(hass, config):
+@asyncio.coroutine
+def async_setup(hass, config):
     """Track states and offer events for locks."""
     component = EntityComponent(
         _LOGGER, DOMAIN, hass, SCAN_INTERVAL, GROUP_NAME_ALL_LOCKS)
-    component.setup(config)
 
-    def handle_lock_service(service):
+    yield from component.async_setup(config)
+
+    @asyncio.coroutine
+    def async_handle_lock_service(service):
         """Handle calls to the lock services."""
-        target_locks = component.extract_from_service(service)
+        target_locks = component.async_extract_from_service(service)
 
         code = service.data.get(ATTR_CODE)
 
-        for item in target_locks:
+        for entity in target_locks:
             if service.service == SERVICE_LOCK:
-                item.lock(code=code)
+                yield from entity.async_lock(code=code)
             else:
-                item.unlock(code=code)
+                yield from entity.async_unlock(code=code)
 
-            if item.should_poll:
-                item.update_ha_state(True)
+        update_tasks = []
+
+        for entity in target_locks:
+            if not entity.should_poll:
+                continue
+
+            update_coro = hass.async_add_job(
+                entity.async_update_ha_state(True))
+            if hasattr(entity, 'async_update'):
+                update_tasks.append(update_coro)
+            else:
+                yield from update_coro
+
+        if update_tasks:
+            yield from asyncio.wait(update_tasks, loop=hass.loop)
+
+    descriptions = yield from hass.loop.run_in_executor(
+        None, load_yaml_config_file, os.path.join(
+            os.path.dirname(__file__), 'services.yaml'))
+
+    hass.services.async_register(
+        DOMAIN, SERVICE_UNLOCK, async_handle_lock_service,
+        descriptions.get(SERVICE_UNLOCK), schema=LOCK_SERVICE_SCHEMA)
+    hass.services.async_register(
+        DOMAIN, SERVICE_LOCK, async_handle_lock_service,
+        descriptions.get(SERVICE_LOCK), schema=LOCK_SERVICE_SCHEMA)
 
-    descriptions = load_yaml_config_file(
-        os.path.join(os.path.dirname(__file__), 'services.yaml'))
-    hass.services.register(DOMAIN, SERVICE_UNLOCK, handle_lock_service,
-                           descriptions.get(SERVICE_UNLOCK),
-                           schema=LOCK_SERVICE_SCHEMA)
-    hass.services.register(DOMAIN, SERVICE_LOCK, handle_lock_service,
-                           descriptions.get(SERVICE_LOCK),
-                           schema=LOCK_SERVICE_SCHEMA)
     return True
 
 
 class LockDevice(Entity):
     """Representation of a lock."""
 
+    @property
+    def changed_by(self):
+        """Last change triggered by."""
+        return None
+
     # pylint: disable=no-self-use
     @property
     def code_format(self):
@@ -116,10 +143,26 @@ class LockDevice(Entity):
         """Lock the lock."""
         raise NotImplementedError()
 
+    def async_lock(self, **kwargs):
+        """Lock the lock.
+
+        This method must be run in the event loop and returns a coroutine.
+        """
+        return self.hass.loop.run_in_executor(
+            None, ft.partial(self.lock, **kwargs))
+
     def unlock(self, **kwargs):
         """Unlock the lock."""
         raise NotImplementedError()
 
+    def async_unlock(self, **kwargs):
+        """Unlock the lock.
+
+        This method must be run in the event loop and returns a coroutine.
+        """
+        return self.hass.loop.run_in_executor(
+            None, ft.partial(self.unlock, **kwargs))
+
     @property
     def state_attributes(self):
         """Return the state attributes."""
@@ -127,6 +170,7 @@ class LockDevice(Entity):
             return None
         state_attr = {
             ATTR_CODE_FORMAT: self.code_format,
+            ATTR_CHANGED_BY: self.changed_by
         }
         return state_attr
 
