@@ -5,20 +5,20 @@ For more details about this platform, please refer to the documentation
 https://home-assistant.io/components/demo/
 """
 from homeassistant.components.lock import LockDevice
-from homeassistant.const import STATE_LOCKED, STATE_UNLOCKED
+from homeassistant.const import (STATE_LOCKED, STATE_UNLOCKED)
 
 
 # pylint: disable=unused-argument
-def setup_platform(hass, config, add_devices_callback, discovery_info=None):
-    """Setup the demo lock platform."""
-    add_devices_callback([
+def setup_platform(hass, config, add_devices, discovery_info=None):
+    """Setup the Demo lock platform."""
+    add_devices([
         DemoLock('Front Door', STATE_LOCKED),
         DemoLock('Kitchen Door', STATE_UNLOCKED)
     ])
 
 
 class DemoLock(LockDevice):
-    """Representation of a demo lock."""
+    """Representation of a Demo lock."""
 
     def __init__(self, name, state):
         """Initialize the lock."""
@@ -43,9 +43,9 @@ class DemoLock(LockDevice):
     def lock(self, **kwargs):
         """Lock the device."""
         self._state = STATE_LOCKED
-        self.update_ha_state()
+        self.schedule_update_ha_state()
 
     def unlock(self, **kwargs):
         """Unlock the device."""
         self._state = STATE_UNLOCKED
-        self.update_ha_state()
+        self.schedule_update_ha_state()
