@@ -4,32 +4,50 @@ Event parser and human readable log generator.
 For more details about this component, please refer to the documentation at
 https://home-assistant.io/components/logbook/
 """
+import asyncio
 import logging
-import re
 from datetime import timedelta
 from itertools import groupby
 
 import voluptuous as vol
 
+from homeassistant.core import callback
 import homeassistant.helpers.config_validation as cv
 import homeassistant.util.dt as dt_util
-from homeassistant.components import recorder, sun
+from homeassistant.components import sun
+from homeassistant.components.frontend import register_built_in_panel
 from homeassistant.components.http import HomeAssistantView
 from homeassistant.const import (EVENT_HOMEASSISTANT_START,
                                  EVENT_HOMEASSISTANT_STOP, EVENT_STATE_CHANGED,
-                                 STATE_NOT_HOME, STATE_OFF, STATE_ON)
-from homeassistant.core import DOMAIN as HA_DOMAIN
-from homeassistant.core import State
-from homeassistant.helpers import template
-from homeassistant.helpers.entity import split_entity_id
+                                 STATE_NOT_HOME, STATE_OFF, STATE_ON,
+                                 ATTR_HIDDEN, HTTP_BAD_REQUEST)
+from homeassistant.core import State, split_entity_id, DOMAIN as HA_DOMAIN
 
 DOMAIN = "logbook"
-DEPENDENCIES = ['recorder', 'http']
-
-URL_LOGBOOK = re.compile(r'/api/logbook(?:/(?P<date>\d{4}-\d{1,2}-\d{1,2})|)')
+DEPENDENCIES = ['recorder', 'frontend']
 
 _LOGGER = logging.getLogger(__name__)
 
+CONF_EXCLUDE = 'exclude'
+CONF_INCLUDE = 'include'
+CONF_ENTITIES = 'entities'
+CONF_DOMAINS = 'domains'
+
+CONFIG_SCHEMA = vol.Schema({
+    DOMAIN: vol.Schema({
+        CONF_EXCLUDE: vol.Schema({
+            vol.Optional(CONF_ENTITIES, default=[]): cv.entity_ids,
+            vol.Optional(CONF_DOMAINS, default=[]): vol.All(cv.ensure_list,
+                                                            [cv.string])
+        }),
+        CONF_INCLUDE: vol.Schema({
+            vol.Optional(CONF_ENTITIES, default=[]): cv.entity_ids,
+            vol.Optional(CONF_DOMAINS, default=[]): vol.All(cv.ensure_list,
+                                                            [cv.string])
+        })
+    }),
+}, extra=vol.ALLOW_EXTRA)
+
 EVENT_LOGBOOK_ENTRY = 'logbook_entry'
 
 GROUP_BY_MINUTES = 15
@@ -41,13 +59,18 @@ ATTR_ENTITY_ID = 'entity_id'
 
 LOG_MESSAGE_SCHEMA = vol.Schema({
     vol.Required(ATTR_NAME): cv.string,
-    vol.Required(ATTR_MESSAGE): cv.string,
+    vol.Required(ATTR_MESSAGE): cv.template,
     vol.Optional(ATTR_DOMAIN): cv.slug,
     vol.Optional(ATTR_ENTITY_ID): cv.entity_id,
 })
 
 
 def log_entry(hass, name, message, domain=None, entity_id=None):
+    """Add an entry to the logbook."""
+    hass.add_job(async_log_entry, hass, name, message, domain, entity_id)
+
+
+def async_log_entry(hass, name, message, domain=None, entity_id=None):
     """Add an entry to the logbook."""
     data = {
         ATTR_NAME: name,
@@ -58,11 +81,12 @@ def log_entry(hass, name, message, domain=None, entity_id=None):
         data[ATTR_DOMAIN] = domain
     if entity_id is not None:
         data[ATTR_ENTITY_ID] = entity_id
-    hass.bus.fire(EVENT_LOGBOOK_ENTRY, data)
+    hass.bus.async_fire(EVENT_LOGBOOK_ENTRY, data)
 
 
 def setup(hass, config):
     """Listen for download events to download files."""
+    @callback
     def log_message(service):
         """Handle sending notification message service calls."""
         message = service.data[ATTR_MESSAGE]
@@ -70,10 +94,14 @@ def setup(hass, config):
         domain = service.data.get(ATTR_DOMAIN)
         entity_id = service.data.get(ATTR_ENTITY_ID)
 
-        message = template.render(hass, message)
-        log_entry(hass, name, message, domain, entity_id)
+        message.hass = hass
+        message = message.async_render()
+        async_log_entry(hass, name, message, domain, entity_id)
 
-    hass.wsgi.register_view(LogbookView)
+    hass.http.register_view(LogbookView(config.get(DOMAIN, {})))
+
+    register_built_in_panel(hass, 'logbook', 'Logbook',
+                            'mdi:format-list-bulleted-type')
 
     hass.services.register(DOMAIN, 'log', log_message,
                            schema=LOG_MESSAGE_SCHEMA)
@@ -85,31 +113,36 @@ class LogbookView(HomeAssistantView):
 
     url = '/api/logbook'
     name = 'api:logbook'
-    extra_urls = ['/api/logbook/<date:date>']
+    extra_urls = ['/api/logbook/{datetime}']
 
-    def get(self, request, date=None):
+    def __init__(self, config):
+        """Initilalize the logbook view."""
+        self.config = config
+
+    @asyncio.coroutine
+    def get(self, request, datetime=None):
         """Retrieve logbook entries."""
-        if date:
-            start_day = dt_util.start_of_local_day(date)
+        if datetime:
+            datetime = dt_util.parse_datetime(datetime)
+
+            if datetime is None:
+                return self.json_message('Invalid datetime', HTTP_BAD_REQUEST)
         else:
-            start_day = dt_util.start_of_local_day()
+            datetime = dt_util.start_of_local_day()
 
-        start_day = dt_util.as_utc(start_day)
+        start_day = dt_util.as_utc(datetime)
         end_day = start_day + timedelta(days=1)
+        hass = request.app['hass']
 
-        events = recorder.get_model('Events')
-        query = recorder.query('Events').filter(
-            (events.time_fired > start_day) &
-            (events.time_fired < end_day))
-        events = recorder.execute(query)
-
+        events = yield from hass.loop.run_in_executor(
+            None, _get_events, hass, start_day, end_day)
+        events = _exclude_events(events, self.config)
         return self.json(humanify(events))
 
 
 class Entry(object):
     """A human readable version of the log."""
 
-    # pylint: disable=too-many-arguments, too-few-public-methods
     def __init__(self, when=None, name=None, message=None, domain=None,
                  entity_id=None):
         """Initialize the entry."""
@@ -137,7 +170,6 @@ def humanify(events):
      - if 2+ sensor updates in GROUP_BY_MINUTES, show last
      - if home assistant stop and start happen in same minute call it restarted
     """
-    # pylint: disable=too-many-branches
     # Group events in batches of GROUP_BY_MINUTES
     for _, g_events in groupby(
             events,
@@ -179,10 +211,6 @@ def humanify(events):
         for event in events_batch:
             if event.event_type == EVENT_STATE_CHANGED:
 
-                # Do not report on new entities
-                if 'old_state' not in event.data:
-                    continue
-
                 to_state = State.from_dict(event.data.get('new_state'))
 
                 # If last_changed != last_updated only attributes have changed
@@ -200,6 +228,11 @@ def humanify(events):
                    event != last_sensor_event[to_state.entity_id]:
                     continue
 
+                # Don't show continuous sensor value changes in the logbook
+                if domain == 'sensor' and \
+                   to_state.attributes.get('unit_of_measurement'):
+                    continue
+
                 yield Entry(
                     event.time_fired,
                     name=to_state.name,
@@ -225,7 +258,7 @@ def humanify(events):
                     event.time_fired, "Home Assistant", action,
                     domain=HA_DOMAIN)
 
-            elif event.event_type.lower() == EVENT_LOGBOOK_ENTRY:
+            elif event.event_type == EVENT_LOGBOOK_ENTRY:
                 domain = event.data.get(ATTR_DOMAIN)
                 entity_id = event.data.get(ATTR_ENTITY_ID)
                 if domain is None and entity_id is not None:
@@ -240,10 +273,96 @@ def humanify(events):
                     entity_id)
 
 
+def _get_events(hass, start_day, end_day):
+    """Get events for a period of time."""
+    from homeassistant.components.recorder.models import Events
+    from homeassistant.components.recorder.util import (
+        execute, session_scope)
+
+    with session_scope(hass=hass) as session:
+        query = session.query(Events).order_by(
+            Events.time_fired).filter(
+                (Events.time_fired > start_day) &
+                (Events.time_fired < end_day))
+        return execute(query)
+
+
+def _exclude_events(events, config):
+    """Get lists of excluded entities and platforms."""
+    excluded_entities = []
+    excluded_domains = []
+    included_entities = []
+    included_domains = []
+    exclude = config.get(CONF_EXCLUDE)
+    if exclude:
+        excluded_entities = exclude[CONF_ENTITIES]
+        excluded_domains = exclude[CONF_DOMAINS]
+    include = config.get(CONF_INCLUDE)
+    if include:
+        included_entities = include[CONF_ENTITIES]
+        included_domains = include[CONF_DOMAINS]
+
+    filtered_events = []
+    for event in events:
+        domain, entity_id = None, None
+
+        if event.event_type == EVENT_STATE_CHANGED:
+            to_state = State.from_dict(event.data.get('new_state'))
+            # Do not report on new entities
+            if event.data.get('old_state') is None:
+                continue
+
+            # Do not report on entity removal
+            if not to_state:
+                continue
+
+            # exclude entities which are customized hidden
+            hidden = to_state.attributes.get(ATTR_HIDDEN, False)
+            if hidden:
+                continue
+
+            domain = to_state.domain
+            entity_id = to_state.entity_id
+
+        elif event.event_type == EVENT_LOGBOOK_ENTRY:
+            domain = event.data.get(ATTR_DOMAIN)
+            entity_id = event.data.get(ATTR_ENTITY_ID)
+
+        if domain or entity_id:
+            # filter if only excluded is configured for this domain
+            if excluded_domains and domain in excluded_domains and \
+                    not included_domains:
+                if (included_entities and entity_id not in included_entities) \
+                        or not included_entities:
+                    continue
+            # filter if only included is configured for this domain
+            elif not excluded_domains and included_domains and \
+                    domain not in included_domains:
+                if (included_entities and entity_id not in included_entities) \
+                        or not included_entities:
+                    continue
+            # filter if included and excluded is configured for this domain
+            elif excluded_domains and included_domains and \
+                    (domain not in included_domains or
+                     domain in excluded_domains):
+                if (included_entities and entity_id not in included_entities) \
+                        or not included_entities or domain in excluded_domains:
+                    continue
+            # filter if only included is configured for this entity
+            elif not excluded_domains and not included_domains and \
+                    included_entities and entity_id not in included_entities:
+                continue
+            # check if logbook entry is excluded for this entity
+            if entity_id in excluded_entities:
+                continue
+        filtered_events.append(event)
+    return filtered_events
+
+
+# pylint: disable=too-many-return-statements
 def _entry_message_from_state(domain, state):
     """Convert a state to a message for the logbook."""
     # We pass domain in so we don't have to split entity_id again
-    # pylint: disable=too-many-return-statements
     if domain == 'device_tracker':
         if state.state == STATE_NOT_HOME:
             return 'is away'
