@@ -9,19 +9,19 @@ from sqlalchemy import (Boolean, Column, DateTime, ForeignKey, Index, Integer,
 from sqlalchemy.ext.declarative import declarative_base
 
 import homeassistant.util.dt as dt_util
-from homeassistant.core import Event, EventOrigin, State
+from homeassistant.core import Event, EventOrigin, State, split_entity_id
 from homeassistant.remote import JSONEncoder
-from homeassistant.helpers.entity import split_entity_id
 
 # SQLAlchemy Schema
 # pylint: disable=invalid-name
 Base = declarative_base()
 
+SCHEMA_VERSION = 2
+
 _LOGGER = logging.getLogger(__name__)
 
 
-class Events(Base):
-    # pylint: disable=too-few-public-methods
+class Events(Base):  # type: ignore
     """Event history data."""
 
     __tablename__ = 'events'
@@ -29,7 +29,7 @@ class Events(Base):
     event_type = Column(String(32), index=True)
     event_data = Column(Text)
     origin = Column(String(32))
-    time_fired = Column(DateTime(timezone=True))
+    time_fired = Column(DateTime(timezone=True), index=True)
     created = Column(DateTime(timezone=True), default=datetime.utcnow)
 
     @staticmethod
@@ -55,25 +55,27 @@ class Events(Base):
             return None
 
 
-class States(Base):
-    # pylint: disable=too-few-public-methods
+class States(Base):   # type: ignore
     """State change history."""
 
     __tablename__ = 'states'
     state_id = Column(Integer, primary_key=True)
     domain = Column(String(64))
-    entity_id = Column(String(64))
+    entity_id = Column(String(255))
     state = Column(String(255))
     attributes = Column(Text)
     event_id = Column(Integer, ForeignKey('events.event_id'))
     last_changed = Column(DateTime(timezone=True), default=datetime.utcnow)
-    last_updated = Column(DateTime(timezone=True), default=datetime.utcnow)
+    last_updated = Column(DateTime(timezone=True), default=datetime.utcnow,
+                          index=True)
     created = Column(DateTime(timezone=True), default=datetime.utcnow)
 
     __table_args__ = (Index('states__state_changes',
                             'last_changed', 'last_updated', 'entity_id'),
                       Index('states__significant_changes',
-                            'domain', 'last_updated', 'entity_id'), )
+                            'domain', 'last_updated', 'entity_id'),
+                      Index('ix_states_entity_id_created',
+                            'entity_id', 'created'),)
 
     @staticmethod
     def from_event(event):
@@ -93,7 +95,8 @@ class States(Base):
         else:
             dbstate.domain = state.domain
             dbstate.state = state.state
-            dbstate.attributes = json.dumps(dict(state.attributes))
+            dbstate.attributes = json.dumps(dict(state.attributes),
+                                            cls=JSONEncoder)
             dbstate.last_changed = state.last_changed
             dbstate.last_updated = state.last_updated
 
@@ -114,8 +117,7 @@ class States(Base):
             return None
 
 
-class RecorderRuns(Base):
-    # pylint: disable=too-few-public-methods
+class RecorderRuns(Base):   # type: ignore
     """Representation of recorder run."""
 
     __tablename__ = 'recorder_runs'
@@ -125,6 +127,8 @@ class RecorderRuns(Base):
     closed_incorrect = Column(Boolean, default=False)
     created = Column(DateTime(timezone=True), default=datetime.utcnow)
 
+    __table_args__ = (Index('ix_recorder_runs_start_end', 'start', 'end'),)
+
     def entity_ids(self, point_in_time=None):
         """Return the entity ids that existed in this run.
 
@@ -152,6 +156,15 @@ class RecorderRuns(Base):
         return self
 
 
+class SchemaChanges(Base):   # type: ignore
+    """Representation of schema version changes."""
+
+    __tablename__ = 'schema_changes'
+    change_id = Column(Integer, primary_key=True)
+    schema_version = Column(Integer)
+    changed = Column(DateTime(timezone=True), default=datetime.utcnow)
+
+
 def _process_timestamp(ts):
     """Process a timestamp into datetime object."""
     if ts is None:
