@@ -7,31 +7,32 @@ https://home-assistant.io/components/sensor.google_travel_time/
 from datetime import datetime
 from datetime import timedelta
 import logging
+
 import voluptuous as vol
 
+from homeassistant.components.sensor import PLATFORM_SCHEMA
 from homeassistant.helpers.entity import Entity
 from homeassistant.const import (
-    CONF_API_KEY, TEMP_CELSIUS, TEMP_FAHRENHEIT,
-    EVENT_HOMEASSISTANT_START, ATTR_LATITUDE, ATTR_LONGITUDE)
-
+    CONF_API_KEY, CONF_NAME, EVENT_HOMEASSISTANT_START, ATTR_LATITUDE,
+    ATTR_LONGITUDE)
 from homeassistant.util import Throttle
 import homeassistant.helpers.config_validation as cv
 import homeassistant.helpers.location as location
 import homeassistant.util.dt as dt_util
 
+REQUIREMENTS = ['googlemaps==2.4.6']
+
 _LOGGER = logging.getLogger(__name__)
 
-REQUIREMENTS = ['googlemaps==2.4.4']
-
-# Return cached results if last update was less then this time ago
-MIN_TIME_BETWEEN_UPDATES = timedelta(minutes=5)
-
-CONF_ORIGIN = 'origin'
 CONF_DESTINATION = 'destination'
-CONF_TRAVEL_MODE = 'travel_mode'
-CONF_OPTIONS = 'options'
 CONF_MODE = 'mode'
-CONF_NAME = 'name'
+CONF_OPTIONS = 'options'
+CONF_ORIGIN = 'origin'
+CONF_TRAVEL_MODE = 'travel_mode'
+
+DEFAULT_NAME = 'Google Travel Time'
+
+MIN_TIME_BETWEEN_UPDATES = timedelta(minutes=5)
 
 ALL_LANGUAGES = ['ar', 'bg', 'bn', 'ca', 'cs', 'da', 'de', 'el', 'en', 'es',
                  'eu', 'fa', 'fi', 'fr', 'gl', 'gu', 'hi', 'hr', 'hu', 'id',
@@ -40,36 +41,34 @@ ALL_LANGUAGES = ['ar', 'bg', 'bn', 'ca', 'cs', 'da', 'de', 'el', 'en', 'es',
                  'sr', 'sv', 'ta', 'te', 'th', 'tl', 'tr', 'uk', 'vi',
                  'zh-CN', 'zh-TW']
 
+AVOID = ['tolls', 'highways', 'ferries', 'indoor']
 TRANSIT_PREFS = ['less_walking', 'fewer_transfers']
+TRANSPORT_TYPE = ['bus', 'subway', 'train', 'tram', 'rail']
+TRAVEL_MODE = ['driving', 'walking', 'bicycling', 'transit']
+TRAVEL_MODEL = ['best_guess', 'pessimistic', 'optimistic']
+UNITS = ['metric', 'imperial']
 
-PLATFORM_SCHEMA = vol.Schema({
-    vol.Required('platform'): 'google_travel_time',
-    vol.Optional(CONF_NAME): vol.Coerce(str),
-    vol.Required(CONF_API_KEY): vol.Coerce(str),
-    vol.Required(CONF_ORIGIN): vol.Coerce(str),
-    vol.Required(CONF_DESTINATION): vol.Coerce(str),
-    vol.Optional(CONF_TRAVEL_MODE):
-        vol.In(["driving", "walking", "bicycling", "transit"]),
+PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend({
+    vol.Required(CONF_API_KEY): cv.string,
+    vol.Required(CONF_DESTINATION): cv.string,
+    vol.Required(CONF_ORIGIN): cv.string,
+    vol.Optional(CONF_NAME): cv.string,
+    vol.Optional(CONF_TRAVEL_MODE): vol.In(TRAVEL_MODE),
     vol.Optional(CONF_OPTIONS, default={CONF_MODE: 'driving'}): vol.All(
         dict, vol.Schema({
-            vol.Optional(CONF_MODE, default='driving'):
-                vol.In(["driving", "walking", "bicycling", "transit"]),
+            vol.Optional(CONF_MODE, default='driving'): vol.In(TRAVEL_MODE),
             vol.Optional('language'): vol.In(ALL_LANGUAGES),
-            vol.Optional('avoid'): vol.In(['tolls', 'highways',
-                                           'ferries', 'indoor']),
-            vol.Optional('units'): vol.In(['metric', 'imperial']),
+            vol.Optional('avoid'): vol.In(AVOID),
+            vol.Optional('units'): vol.In(UNITS),
             vol.Exclusive('arrival_time', 'time'): cv.string,
             vol.Exclusive('departure_time', 'time'): cv.string,
-            vol.Optional('traffic_model'): vol.In(['best_guess',
-                                                   'pessimistic',
-                                                   'optimistic']),
-            vol.Optional('transit_mode'): vol.In(['bus', 'subway', 'train',
-                                                  'tram', 'rail']),
+            vol.Optional('traffic_model'): vol.In(TRAVEL_MODEL),
+            vol.Optional('transit_mode'): vol.In(TRANSPORT_TYPE),
             vol.Optional('transit_routing_preference'): vol.In(TRANSIT_PREFS)
         }))
 })
 
-TRACKABLE_DOMAINS = ["device_tracker", "sensor", "zone"]
+TRACKABLE_DOMAINS = ['device_tracker', 'sensor', 'zone']
 
 
 def convert_time_to_utc(timestr):
@@ -82,20 +81,16 @@ def convert_time_to_utc(timestr):
 
 
 def setup_platform(hass, config, add_devices_callback, discovery_info=None):
-    """Setup the travel time platform."""
-    # pylint: disable=too-many-locals
+    """Setup the Google travel time platform."""
     def run_setup(event):
-        """Delay the setup until home assistant is fully initialized.
+        """Delay the setup until Home Assistant is fully initialized.
 
         This allows any entities to be created already
         """
         options = config.get(CONF_OPTIONS)
 
         if options.get('units') is None:
-            if hass.config.temperature_unit is TEMP_CELSIUS:
-                options['units'] = 'metric'
-            elif hass.config.temperature_unit is TEMP_FAHRENHEIT:
-                options['units'] = 'imperial'
+            options['units'] = hass.config.units.name
 
         travel_mode = config.get(CONF_TRAVEL_MODE)
         mode = options.get(CONF_MODE)
@@ -108,7 +103,7 @@ def setup_platform(hass, config, add_devices_callback, discovery_info=None):
                 options[CONF_MODE] = travel_mode
 
         titled_mode = options.get(CONF_MODE).title()
-        formatted_name = "Google Travel Time - {}".format(titled_mode)
+        formatted_name = "{} - {}".format(DEFAULT_NAME, titled_mode)
         name = config.get(CONF_NAME, formatted_name)
         api_key = config.get(CONF_API_KEY)
         origin = config.get(CONF_ORIGIN)
@@ -124,16 +119,15 @@ def setup_platform(hass, config, add_devices_callback, discovery_info=None):
     hass.bus.listen_once(EVENT_HOMEASSISTANT_START, run_setup)
 
 
-# pylint: disable=too-many-instance-attributes
 class GoogleTravelTimeSensor(Entity):
-    """Representation of a tavel time sensor."""
+    """Representation of a Google travel time sensor."""
 
-    # pylint: disable=too-many-arguments
     def __init__(self, hass, name, api_key, origin, destination, options):
         """Initialize the sensor."""
         self._hass = hass
         self._name = name
         self._options = options
+        self._unit_of_measurement = 'min'
         self._matrix = None
         self.valid_api_connection = True
 
@@ -196,7 +190,7 @@ class GoogleTravelTimeSensor(Entity):
     @property
     def unit_of_measurement(self):
         """Return the unit this state is expressed in."""
-        return "min"
+        return self._unit_of_measurement
 
     @Throttle(MIN_TIME_BETWEEN_UPDATES)
     def update(self):
@@ -244,6 +238,10 @@ class GoogleTravelTimeSensor(Entity):
             self.valid_api_connection = False
             return None
 
+        # Check if the entity has location attributes
+        if location.has_location(entity):
+            return self._get_location_from_attributes(entity)
+
         # Check if device is in a zone
         zone_entity = self._hass.states.get("zone.%s" % entity.state)
         if location.has_location(zone_entity):
@@ -257,10 +255,6 @@ class GoogleTravelTimeSensor(Entity):
         if entity_id.startswith("sensor."):
             return entity.state
 
-        # For everything else look for location attributes
-        if location.has_location(entity):
-            return self._get_location_from_attributes(entity)
-
         # When everything fails just return nothing
         return None
 
