@@ -7,27 +7,38 @@ https://home-assistant.io/components/sun/
 import logging
 from datetime import timedelta
 
-import homeassistant.util as util
+import voluptuous as vol
+
+from homeassistant.const import CONF_ELEVATION
 from homeassistant.helpers.entity import Entity
 from homeassistant.helpers.event import (
     track_point_in_utc_time, track_utc_time_change)
 from homeassistant.util import dt as dt_util
-from homeassistant.const import CONF_ELEVATION
+import homeassistant.helpers.config_validation as cv
+import homeassistant.util as util
 
-REQUIREMENTS = ['astral==1.2']
-DOMAIN = "sun"
-ENTITY_ID = "sun.sun"
-
-STATE_ABOVE_HORIZON = "above_horizon"
-STATE_BELOW_HORIZON = "below_horizon"
-
-STATE_ATTR_NEXT_RISING = "next_rising"
-STATE_ATTR_NEXT_SETTING = "next_setting"
-STATE_ATTR_ELEVATION = "elevation"
-STATE_ATTR_AZIMUTH = "azimuth"
+REQUIREMENTS = ['astral==1.4']
 
 _LOGGER = logging.getLogger(__name__)
 
+DOMAIN = 'sun'
+
+ENTITY_ID = 'sun.sun'
+
+STATE_ABOVE_HORIZON = 'above_horizon'
+STATE_BELOW_HORIZON = 'below_horizon'
+
+STATE_ATTR_AZIMUTH = 'azimuth'
+STATE_ATTR_ELEVATION = 'elevation'
+STATE_ATTR_NEXT_RISING = 'next_rising'
+STATE_ATTR_NEXT_SETTING = 'next_setting'
+
+CONFIG_SCHEMA = vol.Schema({
+    DOMAIN: vol.Schema({
+        vol.Optional(CONF_ELEVATION): cv.positive_int,
+    }),
+}, extra=vol.ALLOW_EXTRA)
+
 
 def is_on(hass, entity_id=None):
     """Test if the sun is currently up based on the statemachine."""
@@ -37,14 +48,20 @@ def is_on(hass, entity_id=None):
 
 
 def next_setting(hass, entity_id=None):
-    """Local datetime object of the next sun setting."""
+    """Local datetime object of the next sun setting.
+
+    Async friendly.
+    """
     utc_next = next_setting_utc(hass, entity_id)
 
     return dt_util.as_local(utc_next) if utc_next else None
 
 
 def next_setting_utc(hass, entity_id=None):
-    """UTC datetime object of the next sun setting."""
+    """UTC datetime object of the next sun setting.
+
+    Async friendly.
+    """
     entity_id = entity_id or ENTITY_ID
 
     state = hass.states.get(ENTITY_ID)
@@ -59,14 +76,20 @@ def next_setting_utc(hass, entity_id=None):
 
 
 def next_rising(hass, entity_id=None):
-    """Local datetime object of the next sun rising."""
+    """Local datetime object of the next sun rising.
+
+    Async friendly.
+    """
     utc_next = next_rising_utc(hass, entity_id)
 
     return dt_util.as_local(utc_next) if utc_next else None
 
 
 def next_rising_utc(hass, entity_id=None):
-    """UTC datetime object of the next sun rising."""
+    """UTC datetime object of the next sun rising.
+
+    Async friendly.
+    """
     entity_id = entity_id or ENTITY_ID
 
     state = hass.states.get(ENTITY_ID)
@@ -208,7 +231,7 @@ class Sun(Entity):
     def point_in_time_listener(self, now):
         """Called when the state of the sun has changed."""
         self.update_as_of(now)
-        self.update_ha_state()
+        self.schedule_update_ha_state()
 
         # Schedule next update at next_change+1 second so sun state has changed
         track_point_in_utc_time(
@@ -218,4 +241,4 @@ class Sun(Entity):
     def timer_update(self, time):
         """Needed to update solar elevation and azimuth."""
         self.update_sun_position(time)
-        self.update_ha_state()
+        self.schedule_update_ha_state()
