@@ -7,6 +7,8 @@ HomeAssistantError will be raised.
 For more details about the Python API, please refer to the documentation at
 https://home-assistant.io/developers/python_api/
 """
+import asyncio
+from concurrent.futures import ThreadPoolExecutor
 from datetime import datetime
 import enum
 import json
@@ -15,13 +17,14 @@ import time
 import threading
 import urllib.parse
 
+from typing import Optional
+
 import requests
 
-import homeassistant.bootstrap as bootstrap
-import homeassistant.core as ha
+from homeassistant import setup, core as ha
 from homeassistant.const import (
     HTTP_HEADER_HA_AUTH, SERVER_PORT, URL_API, URL_API_EVENT_FORWARD,
-    URL_API_EVENTS, URL_API_EVENTS_EVENT, URL_API_SERVICES,
+    URL_API_EVENTS, URL_API_EVENTS_EVENT, URL_API_SERVICES, URL_API_CONFIG,
     URL_API_SERVICES_SERVICE, URL_API_STATES, URL_API_STATES_ENTITY,
     HTTP_HEADER_CONTENT_TYPE, CONTENT_TYPE_JSON)
 from homeassistant.exceptions import HomeAssistantError
@@ -36,13 +39,13 @@ _LOGGER = logging.getLogger(__name__)
 class APIStatus(enum.Enum):
     """Represent API status."""
 
-    # pylint: disable=no-init,invalid-name,too-few-public-methods
+    # pylint: disable=no-init, invalid-name
     OK = "ok"
     INVALID_PASSWORD = "invalid_password"
     CANNOT_CONNECT = "cannot_connect"
     UNKNOWN = "unknown"
 
-    def __str__(self):
+    def __str__(self) -> str:
         """Return the state."""
         return self.value
 
@@ -50,16 +53,23 @@ class APIStatus(enum.Enum):
 class API(object):
     """Object to pass around Home Assistant API location and credentials."""
 
-    # pylint: disable=too-few-public-methods
-    def __init__(self, host, api_password=None, port=None, use_ssl=False):
+    def __init__(self, host: str, api_password: Optional[str]=None,
+                 port: Optional[int]=SERVER_PORT, use_ssl: bool=False) -> None:
         """Initalize the API."""
         self.host = host
-        self.port = port or SERVER_PORT
+        self.port = port
         self.api_password = api_password
-        if use_ssl:
-            self.base_url = "https://{}:{}".format(host, self.port)
+
+        if host.startswith(("http://", "https://")):
+            self.base_url = host
+        elif use_ssl:
+            self.base_url = "https://{}".format(host)
         else:
-            self.base_url = "http://{}:{}".format(host, self.port)
+            self.base_url = "http://{}".format(host)
+
+        if port is not None:
+            self.base_url += ':{}'.format(port)
+
         self.status = None
         self._headers = {
             HTTP_HEADER_CONTENT_TYPE: CONTENT_TYPE_JSON,
@@ -68,14 +78,14 @@ class API(object):
         if api_password is not None:
             self._headers[HTTP_HEADER_HA_AUTH] = api_password
 
-    def validate_api(self, force_validate=False):
+    def validate_api(self, force_validate: bool=False) -> bool:
         """Test if we can communicate with the API."""
         if self.status is None or force_validate:
             self.status = validate_api(self)
 
         return self.status == APIStatus.OK
 
-    def __call__(self, method, path, data=None):
+    def __call__(self, method, path, data=None, timeout=5):
         """Make a call to the Home Assistant API."""
         if data is not None:
             data = json.dumps(data, cls=JSONEncoder)
@@ -85,10 +95,11 @@ class API(object):
         try:
             if method == METHOD_GET:
                 return requests.get(
-                    url, params=data, timeout=5, headers=self._headers)
+                    url, params=data, timeout=timeout, headers=self._headers)
             else:
                 return requests.request(
-                    method, url, data=data, timeout=5, headers=self._headers)
+                    method, url, data=data, timeout=timeout,
+                    headers=self._headers)
 
         except requests.exceptions.ConnectionError:
             _LOGGER.exception("Error connecting to server")
@@ -99,18 +110,20 @@ class API(object):
             _LOGGER.exception(error)
             raise HomeAssistantError(error)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         """Return the representation of the API."""
-        return "API({}, {}, {})".format(
-            self.host, self.api_password, self.port)
+        return "<API({}, password: {})>".format(
+            self.base_url, 'yes' if self.api_password is not None else 'no')
 
 
 class HomeAssistant(ha.HomeAssistant):
     """Home Assistant that forwards work."""
 
-    # pylint: disable=super-init-not-called,too-many-instance-attributes
-    def __init__(self, remote_api, local_api=None):
+    # pylint: disable=super-init-not-called
+    def __init__(self, remote_api, local_api=None, loop=None):
         """Initalize the forward instance."""
+        _LOGGER.warning('Remote instances of Home Assistant are deprecated. '
+                        'Will be removed by 0.43')
         if not remote_api.validate_api():
             raise HomeAssistantError(
                 "Remote API at {}:{} not valid: {}".format(
@@ -118,32 +131,40 @@ class HomeAssistant(ha.HomeAssistant):
 
         self.remote_api = remote_api
 
-        self.pool = pool = ha.create_worker_pool()
+        self.loop = loop or asyncio.get_event_loop()
+        self.executor = ThreadPoolExecutor(max_workers=5)
+        self.loop.set_default_executor(self.executor)
+        self.loop.set_exception_handler(ha.async_loop_exception_handler)
+        self._pending_tasks = []
+        self._pending_sheduler = None
 
-        self.bus = EventBus(remote_api, pool)
-        self.services = ha.ServiceRegistry(self.bus, pool)
-        self.states = StateMachine(self.bus, self.remote_api)
+        self.bus = EventBus(remote_api, self)
+        self.services = ha.ServiceRegistry(self)
+        self.states = StateMachine(self.bus, self.loop, self.remote_api)
         self.config = ha.Config()
+        # This is a dictionary that any component can store any data on.
+        self.data = {}
         self.state = ha.CoreState.not_running
-
+        self.exit_code = None
         self.config.api = local_api
 
     def start(self):
         """Start the instance."""
         # Ensure a local API exists to connect with remote
         if 'api' not in self.config.components:
-            if not bootstrap.setup_component(self, 'api'):
+            if not setup.setup_component(self, 'api'):
                 raise HomeAssistantError(
                     'Unable to setup local API to receive events')
 
         self.state = ha.CoreState.starting
-        ha.create_timer(self)
+        # pylint: disable=protected-access
+        ha._async_create_timer(self)
 
         self.bus.fire(ha.EVENT_HOMEASSISTANT_START,
                       origin=ha.EventOrigin.remote)
 
         # Ensure local HTTP is started
-        self.pool.block_till_done()
+        self.block_till_done()
         self.state = ha.CoreState.running
         time.sleep(0.05)
 
@@ -163,8 +184,6 @@ class HomeAssistant(ha.HomeAssistant):
         self.bus.fire(ha.EVENT_HOMEASSISTANT_STOP,
                       origin=ha.EventOrigin.remote)
 
-        self.pool.stop()
-
         # Disconnect master event forwarding
         disconnect_remote_events(self.remote_api, self.config.api)
         self.state = ha.CoreState.not_running
@@ -173,10 +192,9 @@ class HomeAssistant(ha.HomeAssistant):
 class EventBus(ha.EventBus):
     """EventBus implementation that forwards fire_event to remote API."""
 
-    # pylint: disable=too-few-public-methods
-    def __init__(self, api, pool=None):
+    def __init__(self, api, hass):
         """Initalize the eventbus."""
-        super().__init__(pool)
+        super().__init__(hass)
         self._api = api
 
     def fire(self, event_type, event_data=None, origin=ha.EventOrigin.local):
@@ -199,6 +217,9 @@ class EventForwarder(object):
 
     def __init__(self, hass, restrict_origin=None):
         """Initalize the event forwarder."""
+        _LOGGER.warning('API forwarding is deprecated. '
+                        'Will be removed by 0.43')
+
         self.hass = hass
         self.restrict_origin = restrict_origin
 
@@ -207,34 +228,35 @@ class EventForwarder(object):
         self._targets = {}
 
         self._lock = threading.Lock()
+        self._async_unsub_listener = None
 
-    def connect(self, api):
+    @ha.callback
+    def async_connect(self, api):
         """Attach to a Home Assistant instance and forward events.
 
         Will overwrite old target if one exists with same host/port.
         """
-        with self._lock:
-            if len(self._targets) == 0:
-                # First target we get, setup listener for events
-                self.hass.bus.listen(ha.MATCH_ALL, self._event_listener)
+        if self._async_unsub_listener is None:
+            self._async_unsub_listener = self.hass.bus.async_listen(
+                ha.MATCH_ALL, self._event_listener)
 
-            key = (api.host, api.port)
+        key = (api.host, api.port)
 
-            self._targets[key] = api
+        self._targets[key] = api
 
-    def disconnect(self, api):
+    @ha.callback
+    def async_disconnect(self, api):
         """Remove target from being forwarded to."""
-        with self._lock:
-            key = (api.host, api.port)
+        key = (api.host, api.port)
 
-            did_remove = self._targets.pop(key, None) is None
+        did_remove = self._targets.pop(key, None) is None
 
-            if len(self._targets) == 0:
-                # Remove event listener if no forwarding targets present
-                self.hass.bus.remove_listener(ha.MATCH_ALL,
-                                              self._event_listener)
+        if len(self._targets) == 0:
+            # Remove event listener if no forwarding targets present
+            self._async_unsub_listener()
+            self._async_unsub_listener = None
 
-            return did_remove
+        return did_remove
 
     def _event_listener(self, event):
         """Listen and forward all events."""
@@ -251,9 +273,9 @@ class EventForwarder(object):
 class StateMachine(ha.StateMachine):
     """Fire set events to an API. Uses state_change events to track states."""
 
-    def __init__(self, bus, api):
+    def __init__(self, bus, loop, api):
         """Initalize the statemachine."""
-        super().__init__(None)
+        super().__init__(bus, loop)
         self._api = api
         self.mirror()
 
@@ -286,7 +308,7 @@ class StateMachine(ha.StateMachine):
 class JSONEncoder(json.JSONEncoder):
     """JSONEncoder that supports Home Assistant objects."""
 
-    # pylint: disable=too-few-public-methods,method-hidden
+    # pylint: disable=method-hidden
     def default(self, obj):
         """Convert Home Assistant objects.
 
@@ -294,6 +316,8 @@ class JSONEncoder(json.JSONEncoder):
         """
         if isinstance(obj, datetime):
             return obj.isoformat()
+        elif isinstance(obj, set):
+            return list(obj)
         elif hasattr(obj, 'as_dict'):
             return obj.as_dict()
 
@@ -330,6 +354,8 @@ def validate_api(api):
 
 def connect_remote_events(from_api, to_api):
     """Setup from_api to forward all events to to_api."""
+    _LOGGER.warning('Event forwarding is deprecated. '
+                    'Will be removed by 0.43')
     data = {
         'host': to_api.host,
         'api_password': to_api.api_password,
@@ -355,6 +381,8 @@ def connect_remote_events(from_api, to_api):
 
 def disconnect_remote_events(from_api, to_api):
     """Disconnect forwarding events from from_api to to_api."""
+    _LOGGER.warning('Event forwarding is deprecated. '
+                    'Will be removed by 0.43')
     data = {
         'host': to_api.host,
         'port': to_api.port
@@ -510,12 +538,12 @@ def get_services(api):
         return {}
 
 
-def call_service(api, domain, service, service_data=None):
+def call_service(api, domain, service, service_data=None, timeout=5):
     """Call a service at the remote API."""
     try:
         req = api(METHOD_POST,
                   URL_API_SERVICES_SERVICE.format(domain, service),
-                  service_data)
+                  service_data, timeout=timeout)
 
         if req.status_code != 200:
             _LOGGER.error("Error calling service: %d - %s",
@@ -523,3 +551,23 @@ def call_service(api, domain, service, service_data=None):
 
     except HomeAssistantError:
         _LOGGER.exception("Error calling service")
+
+
+def get_config(api):
+    """Return configuration."""
+    try:
+        req = api(METHOD_GET, URL_API_CONFIG)
+
+        if req.status_code != 200:
+            return {}
+
+        result = req.json()
+        if 'components' in result:
+            result['components'] = set(result['components'])
+        return result
+
+    except (HomeAssistantError, ValueError):
+        # ValueError if req.json() can't parse the JSON
+        _LOGGER.exception("Got unexpected configuration results")
+
+        return {}
